package FDN::Etebac::Sepa;

use strict;
use warnings;
use base 'Exporter';

use Carp;
use FDN::Common::Modules;
use FDN::Common::Config;

our @EXPORT = qw (
	rib_to_bic
	rib_to_iban
);



my $letters;
BEGIN {
	$letters = {};
	my $i;
	foreach $i (0..25) {
		my $l = chr(ord('a')+$i);
		$letters->{$l} = $i+10;
	}
}

sub __char_to_num($) {
	my ( $iban ) = @_;
	$iban = lc($iban);
	$iban =~ s/([a-z])/$letters->{$1}/g;
	return $iban;
}

my $banques;
sub __read_banques {
	my $file = '/usr/share/fdn/data/banques.csv';
	if ( ! -f $file ) {
		confess "File not found: $file (list of banks BIC for France)";
	}
	$banques = {};
	my $line;
	open FILE, '<', $file or die "Cannot read $file: $@";
	while ( $line = <FILE> ) {
		next if $line =~ /^#/;
		next if $line !~ /^FR;([^;]*);([^;]*);(\d+);([^;]*)/;
		my ( $banque, $bic, $code, $ville ) = ( $1, $2, $3, $4 );
		if ( exists $banques->{$code} && $banques->{$code} ne $bic ) {
			confess "J'ai une deuxieme banque avec le meme code guichet: $code in $line.";
		}
		$banques->{$code} = $bic;
	}
	close FILE;
}

sub rib_to_iban {
	my ( $banque, $guichet, $compte, $clefrib ) = @_;
	conduse('Math::BigInt');

	my $clef = $banque.$guichet.$compte.$clefrib.'FR00';
	$clef = __char_to_num($clef);
	$clef = Math::BigInt->new($clef);
	$clef = $clef->bmod(97);
	$clef = 98 - $clef;
	if ( $clef < 10 ) {
		$clef = '0'.$clef;
	}
	return 'FR'.$clef.$banque.$guichet.$compte.$clefrib;
}

sub rib_to_bic {
	my ( $banque, $guichet, $compte, $clef ) = @_;
	if ( ! $banques ) {
		__read_banques();
	}
	if ( ! exists $banques->{$banque} ) {
		return undef;
	}
	return $banques->{$banque};
}

1;
